# -*- coding: utf-8 -*-
###########################################################################
# 
# Eole NG - 2007  
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  http://www.cecill.info/licences/Licence_CeCILL_V2-fr.html
# eole@ac-dijon.fr 
#  
###########################################################################

"""classificateur bayesien

Ce classificateur 

- collecte des mots clefs à partir des répertoires de données
 (par exemple :  *porn* et *noporn*)
- renvoie un coefficient sur la page web ou le contenu parsé

guesser.train('french', 'le la les du un une je il elle de en')

"""

from os import path as osp
from glob import glob
import re
from reverend.thomas import Bayes

from filtrage.config import (GUESS_DATA, BAYES_DATABASE,
                             KEYWORDS, TRAIN_MODE)
from filtrage.errors import DataError
from filtrage.tools import filter_meta, beautiful_soup, normalize_string, average

class Guesser:
    def  __init__(self, filename='guesser.dat'):
        """
        filename: nom long du fichier bayesien
        keywords: liste de mots-clefs (thèmes de filtrage)
        """
        self.guesser = Bayes()
        self.filename = filename

    def train(self, dirname, keyword, mode='meta'):
        """charge tous les fichiers d'un repertoire
        :dirname: nom du repertoire
        :keyword: mot clef associe a ce repertoire"""

        dirname = osp.abspath(osp.normpath(osp.expanduser(dirname)))

        if not osp.isdir(dirname):
            raise Exception, "un repertoire est requis, %s" % dirname

        for filename in glob(osp.join(dirname, "*.*")):
            fh = file(filename, 'r')
            content = fh.read()

            # les différents modes de filtrage possibles
            if mode == 'meta':
                content = filter_meta(content)
            elif mode == 'html':
                content = beautiful_soup(content)
            else:
                # mode txt
                content = normalize_string(content)

            self.guesser.train(keyword, content)
            fh.close()

    def save(self, filename=None):
        """enregistrement de la base bayesienne
        """
        if not filename:
            filename = self.filename
        self.guesser.save(filename)

    def load(self, filename=None):
        "charge un fichier bayesien (en extension .bay)"
        if filename:
            self.filename = filename
        try:
            self.guesser.load(self.filename)
        except:
            raise DataError, "le fichier de donnees de filtrage est inexistant ou bien corrompu"
        
    def guess_line(self, string, mode='meta'):
        """affecte une note de filtrage à la ligne string
        suivant le mode choisi"""
        if mode == 'meta':
            content = filter_meta(string)
        elif mode == 'html':
            content = beautiful_soup(string)
        elif mode == 'weight':
            # FIXME : pour l'instant, seul le body et le meta 
            guess_meta = self.guesser.guess(filter_meta(string))
            guess_body = self.guesser.guess(beautiful_soup(string))
            return average(guess_meta, guess_body)
        else:
            # mode txt
            content = normalize_string(string)

        return self.guesser.guess(content)

#     def guess_file(self, filename):
#         """ouvre un fichier et teste son contenu"""
#         filename = osp.abspath(osp.normpath(osp.expanduser(filename)))
#         if not osp.isfile(filename):
#             raise Exception, "nom de fichier : %s" % filename
#         fh = file(filename, 'r')
#         string = fh.read()
#         fh.close()
#         return self.guess_line(string)

def train_guesser(keywords=KEYWORDS, filename=BAYES_DATABASE, guess_data=GUESS_DATA,
                  mode=TRAIN_MODE):
    """fabrique le classificateur suivant les données
    de configuration et l'enregistre
    chaque nom de répertoire doit avoir le même nom que le nom des mots clefs
    """
    # entrainement sur tous les mot-clefs
    g = Guesser(filename=filename)
    print "creation de la base..."
    for key in keywords:
        print "le mot clef est : ", key
        # creation des bases
        print osp.join(guess_data, key)
        g.train(osp.join(guess_data, key), key, mode=mode)

    # enregistrement de la base 
    g.save(filename)
